`timescale 1ps/1ps

//*****************************************************************************
//
//  Copyright(c) 2016 Synaptic Laboratories Limited (Synaptic Labs)
//
//  All rights reserved.
//
//  This document contains confidential and proprietary product to Synaptic Labs.
//
//  No part of this file may be copied, reproduced, disclosed or provided to
//  third parties without prior written consent of Synaptic Labs.
//
//  Synaptic Labs reserves the right to make changes in the content of this
//  file without prior notice.
//
//*****************************************************************************
//
//*****************************************************************************
//
//  THIS Verilog MODULE IS INTENDED FOR USE IN COMMERCIAL PRODUCTS.
//
//*****************************************************************************
//
//*****************************************************************************
//
//  Title          :  SLL HyperBus controller
//  Project        :  N/A
//
//  File           :  sll_hyperbus_controller_top.v
//  Author         :  Mark Bonnici
//
//  Email          :  mark.bonnici@synaptic-labs.com
//  Revision date  :  Jan/15/2017
//
//  Description    :  SLL  HyperBus controller top module
//
//
//  Known issues   :  None
//
//  References     :
//
//*********************************************************************************************************************************
//  Revision     Date         Author         Comments
//
//  Version1.0   15/Jan/2017   Mark     Initial Release
//  Version2.9   09/Mar/2017   Mark     Added HB_RSTn to  port list
//  Version 2.10 22/Mar/2017   Mark     Addded g_device_family parameter
//*********************************************************************************************************************************

module sll_hyperbus_controller_top (
    in_clk
   ,i_ext_rstn  
    
   ,o_av_out_clk
   ,o_av_out_rstn
    
   //outptu clocks 
   ,i_iavs0_clk
   ,i_iavs0_rstn    

   //
   //Ingress Read/write data port
   //  
   ,i_iavs0_do_rd
   ,i_iavs0_do_wr
   ,i_iavs0_addr
   ,i_iavs0_byteenable
   ,i_iavs0_burstcount
   ,i_iavs0_wdata
   ,o_iavs0_wait_request
   ,o_iavs0_rdata_valid
   ,o_iavs0_rdata
   ,o_iavs0_resp

    //
    //Ingress Register Read/write data port
    //
    ,i_iavsr_do_rd
    ,i_iavsr_do_wr
    ,i_iavsr_addr
    ,i_iavsr_wdata
    ,o_iavsr_rdata
    ,o_iavsr_irq


//Hyper Bus Device Siganls
	 ,HB_CLK0
	 ,HB_CLK0n
	 ,HB_CLK1
	 ,HB_CLK1n
	 ,HB_CSn
	 ,HB_RWDS    
	 ,HB_WPn
	 ,HB_dq
	 ,HB_RSTn
	 ,HB_RSTOn
	 ,HB_INTn
   );                
 parameter g_input_clk_freq_in_mhz   = 50;    //50  Mhz
 parameter g_hyperbus_freq_in_mhz    = 125;   //125 Mhz
 parameter g_iavs_freq_in_mhz        = 125;   //125 Mhz

 parameter g_include_reg_avalon     = 1;       //Include Avalon register space
 parameter g_iavsr_addr_width       = 3;       //register address width 
    
 parameter g_iavs0_addr_width        = 28;      //Address width
 parameter g_iavs0_data_width        = 32;      //Data width
 parameter g_iavs0_av_numsymbols     = 4;       //number of bytes in a word
 parameter g_iavs0_linewrap_burst    = 0;
 parameter g_iavs0_burstonboundaries = 1;       //bursts on burst boundaries
 parameter g_iavs0_burstcount_width  = 4;       //burst count
 parameter g_iavs0_register_rdata    = 0;       //register rdata 
 parameter g_iavs0_register_wdata    = 0;       //register wdata 

 parameter g_same_iavs_hyperbus_clk  = 1;

 parameter g_num_chipselect          = 2;       //Number of chips selects (only 1 or 2 are valid)
 
   //g_dev_xx_config format : Hyperbus Register config[31:16], memory size in mb (log2)[15:4], dev type[3:0] 
 parameter g_dev0_config             = 32'h8fB70060;     
 parameter g_dev1_config             = 32'h8f170031;

   //g_dev_xx_timing format : Tacc[23:16]  T_RWR[15:12], T_CSH[12:8], T_CSS[7:4], T_CSHI[3:0]
 parameter g_dev0_timing             = 32'h00050001; 
 parameter g_dev1_timing             = 32'h00064001;
 
 parameter g_dqin_rdata_width        = 8;
                                     
 parameter g_rst_mode                = 0;       //async reset (0) synct reset (1)

parameter g_device_family              = "MAX 10";   

// global signals
input                                in_clk;
input                                i_ext_rstn;

//output clocks and resets
output                               o_av_out_clk;
output                               o_av_out_rstn;
   
//output clocks and resets
input                                i_iavs0_clk;
input                                i_iavs0_rstn;

//
//Register Read/write data port
//
input                                i_iavsr_do_rd;
input                                i_iavsr_do_wr;
input [g_iavsr_addr_width-1:0]       i_iavsr_addr ;
input [g_iavs0_data_width-1:0]       i_iavsr_wdata;
output  [g_iavs0_data_width-1:0]     o_iavsr_rdata ;
output                               o_iavsr_irq;
   
//
// Read/write data port
//
input                                i_iavs0_do_rd;
input                                i_iavs0_do_wr;
input [g_iavs0_addr_width-1:0]       i_iavs0_addr ;
input [g_iavs0_av_numsymbols-1:0]    i_iavs0_byteenable;
input [g_iavs0_data_width-1:0]       i_iavs0_wdata;
input [g_iavs0_burstcount_width-1:0] i_iavs0_burstcount;
output                               o_iavs0_wait_request;
output                               o_iavs0_rdata_valid;
output  [g_iavs0_data_width-1:0]     o_iavs0_rdata ;
output [1:0]                         o_iavs0_resp;
	
//	
//Hyper Bus Device Siganls
//
input                                HB_RSTOn;
input                                HB_INTn;
output wire                          HB_RSTn;
output wire	                         HB_WPn;
output wire	                         HB_CLK0n;
output wire	                         HB_CLK1;
output wire	                         HB_CLK1n;
output wire	                         HB_CLK0;
output wire	 [g_num_chipselect-1:0]  HB_CSn;
inout  wire	                         HB_RWDS;
inout  wire	 [7:0]                   HB_dq;
	
  // ------------------------------------------------------------
  // hyperbus controller top
  // ------------------------------------------------------------
  wire                               o_iavs0_wait_request;
  wire                               o_iavs0_rdata_valid;
  wire  [g_iavs0_data_width-1:0]     o_iavs0_rdata ;
  wire [1:0]                         o_iavs0_resp;              
  
  wire                               hyperbus_controller_dq_in_en;
  wire                               hyperbus_controller_ck_en;       
  wire  [g_num_chipselect-1:0]       hyperbus_controller_csn_en;     
  wire                               hyperbus_controller_rsto_n;  
  wire                               hyperbus_controller_wp_n  ;
  wire                               hyperbus_controller_hwreset_n;
  wire                               hyperbus_controller_ienon;     
  wire                               hyperbus_controller_csn_d;   
  wire                               hyperbus_controller_rds_clk;    
  wire                               hyperbus_controller_rwds_in;     
  wire                               hyperbus_controller_rwds_io_tri; 
  wire                               hyperbus_controller_wds0;        
  wire                               hyperbus_controller_wds1;        
  wire                               hyperbus_controller_wds_en;      
  wire [g_dqin_rdata_width-1:0]      hyperbus_controller_dq_in;       
  wire                               hyperbus_controller_dq_io_tri;   
  wire                               hyperbus_controller_dq_out_en;   
  wire [7:0]                         hyperbus_controller_dq_out0;     
  wire [7:0]                         hyperbus_controller_dq_out1;     


  // ------------------------------------------------------------
  // output pas 
  // ------------------------------------------------------------
  wire rstn_and_pll_locked;
  
   sll_ca_hbc_t001_io #(
      .g_input_clk_freq_in_mhz (g_input_clk_freq_in_mhz )
     ,.g_hyperbus_freq_in_mhz  (g_hyperbus_freq_in_mhz  )
     ,.g_core_freq_in_mhz      (g_iavs_freq_in_mhz      )
     ,.g_dqin_rdata_width      (g_dqin_rdata_width      )
     ,.g_num_chipselect        (g_num_chipselect        )
     ,.g_device_family         (g_device_family         )
   ) U_IO (
   	 .in_clk       (in_clk)
   	,.ext_nrst     (i_ext_rstn)

    ,.clk_0        (clk_0                           )
    ,.reset_n      (rstn_and_pll_locked             )
    
    ,.clk_core     (clk_core                        )
   //Hyper Bus Cotroller Signals
     ,.ck_en       (hyperbus_controller_ck_en       )
     ,.csn_en      (hyperbus_controller_csn_en      )
     ,.csn_d       (hyperbus_controller_csn_d       )
     ,.dq_in_en    (hyperbus_controller_dq_in_en    )
     ,.rds_clk     (hyperbus_controller_rds_clk     )
     ,.rwds_in     (hyperbus_controller_rwds_in     )
     ,.rwds_io_tri (hyperbus_controller_rwds_io_tri )
     ,.wds0        (hyperbus_controller_wds0        )
     ,.wds1        (hyperbus_controller_wds1        )
     ,.wds_en      (hyperbus_controller_wds_en      )
     ,.dq_in       (hyperbus_controller_dq_in       )
     ,.dq_io_tri   (hyperbus_controller_dq_io_tri   )
     ,.dq_out_en   (hyperbus_controller_dq_out_en   )
     ,.dq_out0     (hyperbus_controller_dq_out0     )
     ,.dq_out1     (hyperbus_controller_dq_out1     )
         
   //Hyper Bus Device Siganls
   	,.HB_CLK0n     (HB_CLK0n)
   	,.HB_CLK0      (HB_CLK0 )
   	,.HB_CLK1      (HB_CLK1 )
   	,.HB_CLK1n     (HB_CLK1n)
   	,.HB_CSn       (HB_CSn  )
   	,.HB_RWDS      (HB_RWDS )
   	,.HB_dq        (HB_dq   )
   );
   assign HB_WPn  = hyperbus_controller_wp_n;
   assign HB_RSTn = hyperbus_controller_hwreset_n;   
  
  assign o_av_out_clk     = (g_same_iavs_hyperbus_clk) ? clk_0 : clk_core;
  assign o_av_out_rstn    = rstn_and_pll_locked;
  
	assign hyperbus_controller_rsto_n = HB_RSTOn;
	assign hyperbus_controller_ienon  = HB_INTn;   
 
	sll_ca_hbc_t001 #(
      .g_iavs0_addr_width       (g_iavs0_addr_width       )     //Address width
     ,.g_iavs0_data_width       (g_iavs0_data_width       )     //Data width
     ,.g_iavs0_av_numsymbols    (g_iavs0_av_numsymbols    )     //number of bytes in a word
     ,.g_iavs0_linewrap_burst   (g_iavs0_linewrap_burst   )
     ,.g_iavs0_burstonboundaries(g_iavs0_burstonboundaries)
     ,.g_iavs0_burstcount_width (g_iavs0_burstcount_width )     //burst count
     ,.g_iavs0_register_rdata   (g_iavs0_register_rdata   )
     ,.g_iavs0_register_wdata   (g_iavs0_register_wdata   )
     
     ,.g_include_reg_avalon     (g_include_reg_avalon     )
     ,.g_iavsr_addr_width       (g_iavsr_addr_width       )
     
     ,.g_same_iavs_hyperbus_clk (g_same_iavs_hyperbus_clk )
     ,.g_dqin_rdata_width       (g_dqin_rdata_width       )

     ,.g_num_chipselect         (g_num_chipselect)
         
   //g_dev_xx_config format : Hyperbus Register config[31:16], memory size in mb (log2)[15:4], dev type[3:0] 
     ,.g_dev0_config            (g_dev0_config)
     ,.g_dev1_config            (g_dev1_config)

   //g_dev_xx_timing format :Tacc[23:16]  T_RWR[15:12], T_CSH[12:8], T_CSS[7:4], T_CSHI[3:0]
     ,.g_dev0_timing            (g_dev0_timing)
     ,.g_dev1_timing            (g_dev1_timing)

     ,.g_rst_mode               (g_rst_mode   )       //async reset (0) synct reset (1)
	  ) U_HBC (
     .hyperbus_clk              (clk_0        )

	  ,.i_iavs0_clk               (i_iavs0_clk  )    
    ,.i_iavs0_rstn              (i_iavs0_rstn )

    //
    //Ingress Read/write data port
    //
    ,.i_iavs0_do_rd             (i_iavs0_do_rd       )
    ,.i_iavs0_do_wr             (i_iavs0_do_wr       )
    ,.i_iavs0_addr              (i_iavs0_addr        )
    ,.i_iavs0_byteenable        (i_iavs0_byteenable  )
    ,.i_iavs0_burstcount        (i_iavs0_burstcount  )
    ,.i_iavs0_wdata             (i_iavs0_wdata       )
    ,.o_iavs0_wait_request      (o_iavs0_wait_request)
    ,.o_iavs0_rdata_valid       (o_iavs0_rdata_valid )
    ,.o_iavs0_rdata             (o_iavs0_rdata       )
    ,.o_iavs0_resp              (o_iavs0_resp        )  

    //
    //Register Read/write data port
    //
    ,.i_iavsr_do_rd             (i_iavsr_do_rd       )
    ,.i_iavsr_do_wr             (i_iavsr_do_wr       )
    ,.i_iavsr_addr              (i_iavsr_addr        )
    ,.i_iavsr_wdata             (i_iavsr_wdata       )
    ,.o_iavsr_rdata             (o_iavsr_rdata       )
    ,.o_iavsr_irq               (o_iavsr_irq         )
          
    //
    //hyperbus Controller ports
    //
`ifdef CK2
   ,.ck2_en                     ()
`endif
   ,.GPO                        (   )
   ,.IENOn                      (   )

   ,.ck_en                      (hyperbus_controller_ck_en      )
   ,.csn_en                     (hyperbus_controller_csn_en     )
   ,.csn_d                      (hyperbus_controller_csn_d      )
   ,.dq_in                      (hyperbus_controller_dq_in      )
   ,.dq_io_tri                  (hyperbus_controller_dq_io_tri  )
   ,.dq_out0                    (hyperbus_controller_dq_out0    )
   ,.dq_out1                    (hyperbus_controller_dq_out1    )
   ,.dq_out_en                  (hyperbus_controller_dq_out_en  )

   ,.hwreset_n                  (hyperbus_controller_hwreset_n  )
   ,.int_n                      (hyperbus_controller_ienon      )
   ,.dq_in_en                   (hyperbus_controller_dq_in_en   )
   ,.rds_clk                    (hyperbus_controller_rds_clk    )
   ,.rwds_in                    (hyperbus_controller_rwds_in    )
   ,.rsto_n                     (hyperbus_controller_rsto_n     )
   ,.rwds_io_tri                (hyperbus_controller_rwds_io_tri)
   ,.wds0                       (hyperbus_controller_wds0       )
   ,.wds1                       (hyperbus_controller_wds1       )
   ,.wds_en                     (hyperbus_controller_wds_en     )
   ,.wp_n                       (hyperbus_controller_wp_n       )
   ,.reset_n                    (  )
   );



endmodule

